@@ -7,10 +7,12 @@ import flasgger
 
 try:
     from marshmallow import Schema, fields
-    from apispec.ext.marshmallow.swagger import (
-        schema2jsonschema, schema2parameters
-    )
+    from apispec.ext.marshmallow import openapi
     from apispec import APISpec as BaseAPISpec
+
+    openapi_converter = openapi.OpenAPIConverter(openapi_version='2.0')
+    schema2jsonschema = openapi_converter.schema2jsonschema
+    schema2parameters = openapi_converter.schema2parameters
 except ImportError:
     Schema = None
     fields = None
